@@ -13,9 +13,10 @@ def optimize(dsk, keys, **kwargs):
 
     if isinstance(dsk, HighLevelGraph):
         # Think about an API for this.
-        dsk = optimize_read_parquet_getitem(dsk)
-        dsk = optimize_blockwise(dsk, keys=list(core.flatten(keys)))
-        dsk = fuse_roots(dsk, keys=list(core.flatten(keys)))
+        flat_keys = list(core.flatten(keys))
+        dsk = optimize_read_parquet_getitem(dsk, keys=flat_keys)
+        dsk = optimize_blockwise(dsk, keys=flat_keys)
+        dsk = fuse_roots(dsk, keys=flat_keys)
 
     dsk = ensure_dict(dsk)
 
@@ -34,7 +35,7 @@ def optimize(dsk, keys, **kwargs):
     return dsk
 
 
-def optimize_read_parquet_getitem(dsk):
+def optimize_read_parquet_getitem(dsk, keys):
     # find the keys to optimize
     from .io.parquet.core import ParquetSubgraph
 
@@ -60,7 +61,15 @@ def optimize_read_parquet_getitem(dsk):
                 return dsk
 
             if list(block.dsk.values())[0][0] != operator.getitem:
-                # ... where this value is __getitem__
+                # ... where this value is __getitem__...
+                return dsk
+
+            if any(block.output == x[0] for x in keys if isinstance(x, tuple)):
+                # if any(block.output == x[0] for x in keys if isinstance(x, tuple)):
+                # ... but bail on the optimization if the getitem is what's requested
+                # These keys are structured like [('getitem-<token>', 0), ...]
+                # so we check for the first item of the tuple.
+                # See https://github.com/dask/dask/issues/5893
                 return dsk
 
             block_columns = block.indices[1][0]
