@@ -1,10 +1,11 @@
 from collections import namedtuple
 from operator import add, setitem
+from functools import partial
 import pickle
 from random import random
 import types
 
-from toolz import identity, partial, merge
+from tlz import merge
 import pytest
 
 import dask
@@ -498,15 +499,19 @@ def test_callable_obj():
     assert f().compute() == 2
 
 
+def identity(x):
+    return x
+
+
 def test_name_consistent_across_instances():
     func = delayed(identity, pure=True)
 
     data = {"x": 1, "y": 25, "z": [1, 2, 3]}
-    assert func(data)._key == "identity-84c5e2194036c17d1d97c4e3a2b90482"
+    assert func(data)._key == "identity-02129ed1acaffa7039deee80c5da547c"
 
     data = {"x": 1, 1: "x"}
     assert func(data)._key == func(data)._key
-    assert func(1)._key == "identity-7126728842461bf3d2caecf7b954fa3b"
+    assert func(1)._key == "identity-ca2fae46a3b938016331acac1908ae45"
 
 
 def test_sensitive_to_partials():
@@ -615,3 +620,41 @@ def test_check_meta_flag():
 
     c = from_delayed([da, db], verify_meta=False)
     assert_eq(c, c)
+
+
+def modlevel_eager(x):
+    return x + 1
+
+
+@delayed
+def modlevel_delayed1(x):
+    return x + 1
+
+
+@delayed(pure=False)
+def modlevel_delayed2(x):
+    return x + 1
+
+
+@pytest.mark.parametrize(
+    "f",
+    [
+        delayed(modlevel_eager),
+        pytest.param(modlevel_delayed1, marks=pytest.mark.xfail(reason="#3369")),
+        pytest.param(modlevel_delayed2, marks=pytest.mark.xfail(reason="#3369")),
+    ],
+)
+def test_pickle(f):
+    d = f(2)
+    d = pickle.loads(pickle.dumps(d, protocol=pickle.HIGHEST_PROTOCOL))
+    assert d.compute() == 3
+
+
+@pytest.mark.parametrize(
+    "f", [delayed(modlevel_eager), modlevel_delayed1, modlevel_delayed2]
+)
+def test_cloudpickle(f):
+    cloudpickle = pytest.importorskip("cloudpickle")
+    d = f(2)
+    d = cloudpickle.loads(cloudpickle.dumps(d, protocol=pickle.HIGHEST_PROTOCOL))
+    assert d.compute() == 3
